@@ -1,7 +1,6 @@
 from .__version__ import __version__
-from .mocks import DeprecatedMockTransport as MockTransport
 from .models import MockResponse, Route
-from .router import Router
+from .router import DeprecatedMockTransport as MockTransport, MockRouter, Router
 
 from .api import (  # isort:skip
     mock,
@@ -28,6 +27,7 @@ __all__ = [
     "__version__",
     "MockTransport",
     "MockResponse",
+    "MockRouter",
     "Router",
     "Route",
     "mock",
