@@ -1,18 +1,29 @@
 import inspect
-from functools import wraps
+from functools import update_wrapper
 from types import TracebackType
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union, overload
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    NewType,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    overload,
+)
 from warnings import warn
 
 import httpx
 
-from .mocks import BaseMock, HTTPCoreMock, HTTPCoreMock as DefaultMock
-from .models import CallList, Route, SideEffectError
+from .mocks import Mocker
+from .models import CallList, PassThrough, Route, RouteList, SideEffectError
 from .patterns import Pattern, merge_patterns, parse_url_patterns
 from .types import DefaultType, URLPatternTypes
 
-Default = object
-DEFAULT = Default()
+Default = NewType("Default", object)
+DEFAULT = Default(...)
 
 
 class Router:
@@ -27,7 +38,7 @@ class Router:
         self._assert_all_mocked = assert_all_mocked
         self._bases = parse_url_patterns(base_url, exact=False)
 
-        self.routes: Dict[Union[str, int], Route] = {}
+        self.routes = RouteList()
         self.calls = CallList()
 
         self._snapshots: List[Tuple] = []
@@ -44,12 +55,12 @@ class Router:
         Snapshots current routes and calls state.
         """
         # Snapshot current routes and calls
-        routes = dict(self.routes)
+        routes = RouteList(self.routes)
         calls = CallList(self.calls)
         self._snapshots.append((routes, calls))
 
         # Snapshot each route state
-        for route in routes.values():
+        for route in routes:
             route.snapshot()
 
     def rollback(self) -> None:
@@ -61,12 +72,11 @@ class Router:
 
         # Revert added routes and calls to last snapshot
         routes, calls = self._snapshots.pop()
-        self.routes.clear()
-        self.routes.update(routes)
+        self.routes[:] = routes
         self.calls[:] = calls
 
         # Revert each route state to last snapshot
-        for route in self.routes.values():
+        for route in self.routes:
             route.rollback()
 
     def reset(self) -> None:
@@ -74,16 +84,16 @@ class Router:
         Resets call stats.
         """
         self.calls.clear()
-        for route in self.routes.values():
+        for route in self.routes:
             route.reset()
 
     def assert_all_called(self) -> None:
         assert all(
-            (route.called for route in self.routes.values())
+            (route.called for route in self.routes)
         ), "RESPX: some mocked requests were not called!"
 
-    def __getitem__(self, name: str) -> Optional[Route]:
-        return self.routes.get(name)
+    def __getitem__(self, name: str) -> Route:
+        return self.routes[name]
 
     @overload
     def pop(self, name: str) -> Route:
@@ -115,34 +125,17 @@ class Router:
     def add(self, route: Route, *, name: Optional[str] = None) -> Route:
         """
         Adds a route with optionally given name,
-        replacing any existing route with same pattern.
+        replacing any existing route with same name or pattern.
         """
         if not isinstance(route, Route):
             raise ValueError(
                 f"Invalid route {route!r}, please use respx.route(...).mock(...)"
             )
 
-        # Merge bases
-        route.pattern = merge_patterns(route.pattern, **self._bases)
-        route.name = name
-
-        if route.id in self.routes:
-            route = self.replace(route)
-        else:
-            self.routes[route.id] = route
-
+        route._pattern = merge_patterns(route.pattern, **self._bases)
+        route = self.routes.add(route, name=name)
         return route
 
-    def replace(self, route: Route) -> Route:
-        """
-        Replace existing route with same pattern, in same place (order).
-        """
-        existing_route = self.routes[route.id]
-        existing_route.return_value = route.return_value
-        existing_route.side_effect = route.side_effect
-        existing_route.pass_through(route.is_pass_through)
-        return existing_route
-
     def request(
         self,
         method: str,
@@ -234,7 +227,7 @@ class Router:
         route: Optional[Route] = None
         response: Optional[Union[httpx.Request, httpx.Response]] = None
 
-        for prospect in self.routes.values():
+        for prospect in self.routes:
             response = prospect.match(request)
             if response:
                 route = prospect
@@ -273,11 +266,17 @@ class Router:
 
         return response
 
+    def handler(self, request: httpx.Request) -> httpx.Response:
+        response = self.resolve(request)
+        if response is None:
+            raise PassThrough(
+                f"Request marked to pass through: {request!r}", request=request
+            )
+        return response
+
 
 class MockRouter(Router):
-    Mock: Optional[Type[BaseMock]]
-    handler = Router.resolve
-    _local = False
+    Mocker: Optional[Type[Mocker]]
 
     def __init__(
         self,
@@ -292,10 +291,7 @@ class MockRouter(Router):
             assert_all_mocked=assert_all_mocked,
             base_url=base_url,
         )
-        self.Mock = {
-            "httpcore": HTTPCoreMock,
-            DEFAULT: DefaultMock,
-        }.get(using)
+        self._using = using
 
     def __call__(
         self,
@@ -326,27 +322,32 @@ class MockRouter(Router):
             if assert_all_mocked is not None:
                 settings["assert_all_mocked"] = assert_all_mocked
             respx_mock = self.__class__(**settings)
-            respx_mock._local = True
             return respx_mock
 
+        # Determine if decorated function needs a `respx_mock` instance
+        argspec = inspect.getfullargspec(func)
+        needs_mock_reference = "respx_mock" in argspec.args
+
         # Async Decorator
-        @wraps(func)
         async def async_decorator(*args, **kwargs):
             assert func is not None
-            if self._local:
+            if needs_mock_reference:
                 kwargs["respx_mock"] = self
             async with self:
                 return await func(*args, **kwargs)
 
         # Sync Decorator
-        @wraps(func)
         def sync_decorator(*args, **kwargs):
             assert func is not None
-            if self._local:
+            if "respx_mock" in argspec.args:
                 kwargs["respx_mock"] = self
             with self:
                 return func(*args, **kwargs)
 
+        if not needs_mock_reference:
+            async_decorator = update_wrapper(async_decorator, func)
+            sync_decorator = update_wrapper(sync_decorator, func)
+
         # Dispatch async/sync decorator, depening on decorated function.
         # - Only stage when using global decorator `@respx.mock`
         # - Second stage when using local decorator `@respx.mock(...)`
@@ -370,21 +371,37 @@ class MockRouter(Router):
     async def __aexit__(self, *args: Any) -> None:
         self.__exit__(*args)
 
+    @property
+    def using(self) -> Optional[str]:
+        from respx.mocks import DEFAULT_MOCKER
+
+        if self._using is None:
+            using = None
+        elif self._using is DEFAULT:
+            using = DEFAULT_MOCKER
+        elif isinstance(self._using, str):
+            using = self._using
+        else:
+            raise ValueError(f"Invalid Router `using` kwarg: {self._using!r}")
+
+        return using
+
     def start(self) -> None:
         """
         Register transport, snapshot router and start patching.
         """
         self.snapshot()
-        if self.Mock:
-            self.Mock.register(self)
-            self.Mock.start()
+        self.Mocker = Mocker.registry.get(self.using)
+        if self.Mocker:
+            self.Mocker.register(self)
+            self.Mocker.start()
 
     def stop(self, clear: bool = True, reset: bool = True, quiet: bool = False) -> None:
         """
         Unregister transport and rollback router.
         Stop patching when no registered transports left.
         """
-        unregistered = self.Mock.unregister(self) if self.Mock else True
+        unregistered = self.Mocker.unregister(self) if self.Mocker else True
 
         try:
             if unregistered and not quiet and self._assert_all_called:
@@ -394,8 +411,8 @@ class MockRouter(Router):
                 self.rollback()
             if reset:
                 self.reset()
-            if self.Mock:
-                self.Mock.stop()
+            if self.Mocker:
+                self.Mocker.stop()
 
 
 class DeprecatedMockTransport(MockRouter):
